@@ -3,7 +3,7 @@ from ftplib import FTP, Error, error_perm
 from typing import Any
 
 from ..spec import AbstractBufferedFile, AbstractFileSystem
-from ..utils import infer_storage_options
+from ..utils import infer_storage_options, isfilelike
 
 
 class FTPFileSystem(AbstractFileSystem):
@@ -21,7 +21,7 @@ class FTPFileSystem(AbstractFileSystem):
         password=None,
         acct=None,
         block_size=None,
-        tempdir="/tmp",
+        tempdir=None,
         timeout=30,
         **kwargs,
     ):
@@ -54,13 +54,13 @@ class FTPFileSystem(AbstractFileSystem):
         super(FTPFileSystem, self).__init__(**kwargs)
         self.host = host
         self.port = port
-        self.tempdir = tempdir
+        self.tempdir = tempdir or "/tmp"
         self.cred = username, password, acct
         self.timeout = timeout
         if block_size is not None:
             self.blocksize = block_size
         else:
-            self.blocksize = 2 ** 16
+            self.blocksize = 2**16
         self._connect()
 
     def _connect(self):
@@ -110,7 +110,7 @@ class FTPFileSystem(AbstractFileSystem):
                     if info["type"] == "file":
                         out = [(path, info)]
                 except (Error, IndexError):
-                    raise FileNotFoundError
+                    raise FileNotFoundError(path)
         files = self.dircache.get(path, out)
         if not detail:
             return sorted([fn for fn, details in files])
@@ -130,16 +130,21 @@ class FTPFileSystem(AbstractFileSystem):
         return out
 
     def get_file(self, rpath, lpath, **kwargs):
-        with open(lpath, "wb") as outfile:
+        if isfilelike(lpath):
+            outfile = lpath
+        else:
+            outfile = open(lpath, "wb")
 
-            def cb(x):
-                outfile.write(x)
+        def cb(x):
+            outfile.write(x)
 
-            self.ftp.retrbinary(
-                "RETR %s" % rpath,
-                blocksize=self.blocksize,
-                callback=cb,
-            )
+        self.ftp.retrbinary(
+            "RETR %s" % rpath,
+            blocksize=self.blocksize,
+            callback=cb,
+        )
+        if not isfilelike(lpath):
+            outfile.close()
 
     def cat_file(self, path, start=None, end=None, **kwargs):
         if end is not None:
