@@ -11,11 +11,11 @@ from urllib.parse import urlparse
 import aiohttp
 import requests
 
-from fsspec.asyn import AsyncFileSystem, sync, sync_wrapper
+from fsspec.asyn import AbstractAsyncStreamedFile, AsyncFileSystem, sync, sync_wrapper
 from fsspec.callbacks import _DEFAULT_CALLBACK
 from fsspec.exceptions import FSTimeoutError
 from fsspec.spec import AbstractBufferedFile
-from fsspec.utils import DEFAULT_BLOCK_SIZE, nullcontext, tokenize
+from fsspec.utils import DEFAULT_BLOCK_SIZE, isfilelike, nullcontext, tokenize
 
 from ..caching import AllBytes
 
@@ -104,9 +104,6 @@ class HTTPFileSystem(AsyncFileSystem):
         request_options.pop("skip_instance_cache", None)
         self.kwargs = request_options
 
-        if not asynchronous:
-            sync(self.loop, self.set_session)
-
     @staticmethod
     def close_session(loop, session):
         if loop is not None and loop.is_running():
@@ -212,7 +209,6 @@ class HTTPFileSystem(AsyncFileSystem):
         kw.update(kwargs)
         logger.debug(url)
 
-        # TODO: extract into testable utility function?
         if start is not None or end is not None:
             if start == end:
                 return b""
@@ -227,7 +223,7 @@ class HTTPFileSystem(AsyncFileSystem):
         return out
 
     async def _get_file(
-        self, rpath, lpath, chunk_size=5 * 2 ** 20, callback=_DEFAULT_CALLBACK, **kwargs
+        self, rpath, lpath, chunk_size=5 * 2**20, callback=_DEFAULT_CALLBACK, **kwargs
     ):
         kw = self.kwargs.copy()
         kw.update(kwargs)
@@ -241,18 +237,19 @@ class HTTPFileSystem(AsyncFileSystem):
 
             callback.set_size(size)
             self._raise_not_found_for_status(r, rpath)
-            with open(lpath, "wb") as fd:
-                chunk = True
-                while chunk:
-                    chunk = await r.content.read(chunk_size)
-                    fd.write(chunk)
-                    callback.relative_update(len(chunk))
+            if not isfilelike(lpath):
+                lpath = open(lpath, "wb")
+            chunk = True
+            while chunk:
+                chunk = await r.content.read(chunk_size)
+                lpath.write(chunk)
+                callback.relative_update(len(chunk))
 
     async def _put_file(
         self,
         lpath,
         rpath,
-        chunk_size=5 * 2 ** 20,
+        chunk_size=5 * 2**20,
         callback=_DEFAULT_CALLBACK,
         method="post",
         **kwargs,
@@ -274,11 +271,11 @@ class HTTPFileSystem(AsyncFileSystem):
                 else:
                     callback.set_size(getattr(f, "size", None))
 
-                chunk = f.read(64 * 1024)
+                chunk = f.read(chunk_size)
                 while chunk:
                     yield chunk
                     callback.relative_update(len(chunk))
-                    chunk = f.read(64 * 1024)
+                    chunk = f.read(chunk_size)
 
         kw = self.kwargs.copy()
         kw.update(kwargs)
@@ -360,6 +357,17 @@ class HTTPFileSystem(AsyncFileSystem):
                 self, path, mode=mode, loop=self.loop, session=session, **kw
             )
 
+    async def open_async(self, path, mode="rb", size=None, **kwargs):
+        session = await self.set_session()
+        if size is None:
+            try:
+                size = (await self._info(path, **kwargs))["size"]
+            except FileNotFoundError:
+                pass
+        return AsyncStreamFile(
+            self, path, loop=self.loop, session=session, size=size, **kwargs
+        )
+
     def ukey(self, url):
         """Unique identifier; assume HTTP files are static, unchanging"""
         return tokenize(url, self.kwargs, self.protocol)
@@ -612,7 +620,7 @@ class HTTPFile(AbstractBufferedFile):
                 cl = 0
                 out = []
                 while True:
-                    chunk = await r.content.read(2 ** 20)
+                    chunk = await r.content.read(2**20)
                     # data size unknown, let's read until we have enough
                     if chunk:
                         out.append(chunk)
@@ -626,9 +634,6 @@ class HTTPFile(AbstractBufferedFile):
 
     _fetch_range = sync_wrapper(async_fetch_range)
 
-    def close(self):
-        pass
-
     def __reduce__(self):
         return (
             reopen,
@@ -637,7 +642,7 @@ class HTTPFile(AbstractBufferedFile):
                 self.url,
                 self.mode,
                 self.blocksize,
-                self.cache.name,
+                self.cache.name if self.cache else "none",
                 self.size,
             ),
         )
@@ -690,11 +695,42 @@ class HTTPStreamFile(AbstractBufferedFile):
 
     def close(self):
         asyncio.run_coroutine_threadsafe(self._close(), self.loop)
+        super().close()
 
     def __reduce__(self):
         return reopen, (self.fs, self.url, self.mode, self.blocksize, self.cache.name)
 
 
+class AsyncStreamFile(AbstractAsyncStreamedFile):
+    def __init__(
+        self, fs, url, mode="rb", loop=None, session=None, size=None, **kwargs
+    ):
+        self.url = url
+        self.session = session
+        self.r = None
+        if mode != "rb":
+            raise ValueError
+        self.details = {"name": url, "size": None}
+        self.kwargs = kwargs
+        super().__init__(fs=fs, path=url, mode=mode, cache_type="none")
+        self.size = size
+
+    async def read(self, num=-1):
+        if self.r is None:
+            r = await self.session.get(self.url, **self.kwargs).__aenter__()
+            self.fs._raise_not_found_for_status(r, self.url)
+            self.r = r
+        out = await self.r.content.read(num)
+        self.loc += len(out)
+        return out
+
+    async def close(self):
+        if self.r is not None:
+            self.r.close()
+            self.r = None
+        await super().close()
+
+
 async def get_range(session, url, start, end, file=None, **kwargs):
     # explicit get a range when we know it must be safe
     kwargs = kwargs.copy()
