@@ -1,21 +1,23 @@
-from __future__ import print_function, division, absolute_import
+from __future__ import absolute_import, division, print_function
 
+import bz2
 import gzip
 import os
 import os.path
 import pickle
-import sys
-from contextlib import contextmanager
-from distutils.version import LooseVersion
 import posixpath
+import sys
 import tempfile
+from contextlib import contextmanager
+from pathlib import Path
+from unittest.mock import patch
 
 import pytest
+
 import fsspec
-from unittest.mock import patch
-from fsspec.core import open_files, get_fs_token_paths, OpenFile
-from fsspec.implementations.local import LocalFileSystem, make_path_posix
 from fsspec import compression
+from fsspec.core import OpenFile, get_fs_token_paths, open_files
+from fsspec.implementations.local import LocalFileSystem, make_path_posix
 from fsspec.tests.test_utils import WIN
 
 files = {
@@ -118,6 +120,26 @@ def test_urlpath_expand_read():
         assert len(paths) == 2
 
 
+def test_cats():
+    with filetexts(csv_files, mode="b"):
+        fs = fsspec.filesystem("file")
+        assert fs.cat(".test.fakedata.1.csv") == b"a,b\n" b"1,2\n"
+        out = set(fs.cat([".test.fakedata.1.csv", ".test.fakedata.2.csv"]).values())
+        assert out == {b"a,b\n" b"1,2\n", b"a,b\n" b"3,4\n"}
+        assert fs.cat(".test.fakedata.1.csv", None, None) == b"a,b\n" b"1,2\n"
+        assert fs.cat(".test.fakedata.1.csv", start=1, end=6) == b"a,b\n" b"1,2\n"[1:6]
+        assert fs.cat(".test.fakedata.1.csv", start=-1) == b"a,b\n" b"1,2\n"[-1:]
+        assert (
+            fs.cat(".test.fakedata.1.csv", start=1, end=-2) == b"a,b\n" b"1,2\n"[1:-2]
+        )
+        out = set(
+            fs.cat(
+                [".test.fakedata.1.csv", ".test.fakedata.2.csv"], start=1, end=-1
+            ).values()
+        )
+        assert out == {b"a,b\n" b"1,2\n"[1:-1], b"a,b\n" b"3,4\n"[1:-1]}
+
+
 def test_urlpath_expand_write():
     """Make sure * is expanded in file paths when writing."""
     _, _, paths = get_fs_token_paths("prefix-*.csv", mode="wb", num=2)
@@ -160,9 +182,6 @@ def test_open_files_text_mode(encoding):
 @pytest.mark.parametrize("mode", ["rt", "rb"])
 @pytest.mark.parametrize("fmt", list(compression.compr))
 def test_compressions(fmt, mode, tmpdir):
-    if fmt == "zip" and sys.version_info < (3, 6):
-        pytest.xfail("zip compression requires python3.6 or higher")
-
     tmpdir = str(tmpdir)
     fn = os.path.join(tmpdir, ".tmp.getsize")
     fs = LocalFileSystem()
@@ -309,7 +328,7 @@ def test_globfind_dirs(tmpdir):
 def test_touch(tmpdir):
     import time
 
-    fn = tmpdir + "/in/file"
+    fn = str(tmpdir + "/in/file")
     fs = fsspec.filesystem("file", auto_mkdir=False)
     with pytest.raises(OSError):
         fs.touch(fn)
@@ -323,20 +342,16 @@ def test_touch(tmpdir):
         assert info2["mtime"] > info["mtime"]
 
 
-def test_get_pyarrow_filesystem():
-    pa = pytest.importorskip("pyarrow")
-
-    fs = LocalFileSystem()
-    if LooseVersion(pa.__version__) < LooseVersion("2.0"):
-        assert isinstance(fs, pa.filesystem.FileSystem)
-        assert fs._get_pyarrow_filesystem() is fs
-    else:
-        assert not isinstance(fs, pa.filesystem.FileSystem)
-
-    class UnknownFileSystem(object):
-        pass
-
-    assert not isinstance(UnknownFileSystem(), pa.filesystem.FileSystem)
+def test_touch_truncate(tmpdir):
+    fn = str(tmpdir + "/tfile")
+    fs = fsspec.filesystem("file")
+    fs.touch(fn, truncate=True)
+    fs.pipe(fn, b"a")
+    fs.touch(fn, truncate=True)
+    assert fs.cat(fn) == b""
+    fs.pipe(fn, b"a")
+    fs.touch(fn, truncate=False)
+    assert fs.cat(fn) == b"a"
 
 
 def test_directories(tmpdir):
@@ -347,6 +362,7 @@ def test_directories(tmpdir):
     assert fs.ls(tmpdir, True)[0]["type"] == "directory"
     fs.rmdir(tmpdir + "/dir")
     assert not fs.ls(tmpdir)
+    assert fs.ls(fs.root_marker)
 
 
 def test_file_ops(tmpdir):
@@ -376,6 +392,22 @@ def test_file_ops(tmpdir):
     fs.rm(tmpdir + "/afile3", recursive=True)
     assert not fs.exists(tmpdir + "/afile3")
 
+    files = [tmpdir + "/afile4", tmpdir + "/afile5"]
+    [fs.touch(f) for f in files]
+
+    with pytest.raises(TypeError):
+        fs.rm_file(files)
+    fs.rm(files)
+    assert all(not fs.exists(f) for f in files)
+
+    fs.touch(tmpdir + "/afile6")
+    fs.rm_file(tmpdir + "/afile6")
+    assert not fs.exists(tmpdir + "/afile6")
+
+    # IsADirectoryError raised on Linux, PermissionError on Windows
+    with pytest.raises((IsADirectoryError, PermissionError)):
+        fs.rm_file(tmpdir)
+
     fs.rm(tmpdir, recursive=True)
     assert not fs.exists(tmpdir)
 
@@ -427,29 +459,29 @@ def test_make_path_posix():
     else:
         assert make_path_posix("/a/posix/path") == "/a/posix/path"
         assert make_path_posix("/posix") == "/posix"
-    assert make_path_posix("relpath", sep="/") == posixpath.join(
-        make_path_posix(cwd), "relpath"
-    )
-    assert make_path_posix("rel/path", sep="/") == posixpath.join(
+    assert make_path_posix("relpath") == posixpath.join(make_path_posix(cwd), "relpath")
+    assert make_path_posix("rel/path") == posixpath.join(
         make_path_posix(cwd), "rel/path"
     )
-    assert make_path_posix("C:\\path", sep="\\") == "C:/path"
-    assert (
-        make_path_posix(
-            "\\\\windows-server\\someshare\\path\\more\\path\\dir\\foo.parquet"
+    if WIN:
+        assert make_path_posix("C:\\path") == "C:/path"
+    if WIN:
+        assert (
+            make_path_posix(
+                "\\\\windows-server\\someshare\\path\\more\\path\\dir\\foo.parquet"
+            )
+            == "//windows-server/someshare/path/more/path/dir/foo.parquet"
         )
-        == "//windows-server/someshare/path/more/path/dir/foo.parquet"
-    )
-    assert (
-        make_path_posix(
-            r"\\SERVER\UserHomeFolder$\me\My Documents\project1\data\filen.csv"
+        assert (
+            make_path_posix(
+                r"\\SERVER\UserHomeFolder$\me\My Documents\project1\data\filen.csv"
+            )
+            == "//SERVER/UserHomeFolder$/me/My Documents/project1/data/filen.csv"
         )
-        == "//SERVER/UserHomeFolder$/me/My Documents/project1/data/filen.csv"
-    )
-    assert "/" in make_path_posix("rel\\path", sep="\\")
+    assert "/" in make_path_posix("rel\\path")
 
 
-def test_links(tmpdir):
+def test_linked_files(tmpdir):
     tmpdir = str(tmpdir)
     fn0 = os.path.join(tmpdir, "target")
     fn1 = os.path.join(tmpdir, "link1")
@@ -468,8 +500,12 @@ def test_links(tmpdir):
 
     fs = LocalFileSystem()
     assert fs.info(fn0)["type"] == "file"
-    assert fs.info(fn1)["type"] == "link"
-    assert fs.info(fn2)["type"] == "link"
+    assert fs.info(fn1)["type"] == "file"
+    assert fs.info(fn2)["type"] == "file"
+
+    assert not fs.info(fn0)["islink"]
+    assert fs.info(fn1)["islink"]
+    assert fs.info(fn2)["islink"]
 
     assert fs.info(fn0)["size"] == len(data)
     assert fs.info(fn1)["size"] == len(data)
@@ -484,6 +520,63 @@ def test_links(tmpdir):
         assert f.read() == data
 
 
+def test_linked_files_exists(tmpdir):
+    origin = tmpdir / "original"
+    copy_file = tmpdir / "copy"
+
+    fs = LocalFileSystem()
+    fs.touch(origin)
+
+    try:
+        os.symlink(origin, copy_file)
+    except OSError:
+        if WIN:
+            pytest.xfail("Ran on win without admin permissions")
+        else:
+            raise
+
+    assert fs.exists(copy_file)
+    assert fs.lexists(copy_file)
+
+    os.unlink(origin)
+
+    assert not fs.exists(copy_file)
+    assert fs.lexists(copy_file)
+
+    os.unlink(copy_file)
+
+    assert not fs.exists(copy_file)
+    assert not fs.lexists(copy_file)
+
+
+def test_linked_directories(tmpdir):
+    tmpdir = str(tmpdir)
+
+    subdir0 = os.path.join(tmpdir, "target")
+    subdir1 = os.path.join(tmpdir, "link1")
+    subdir2 = os.path.join(tmpdir, "link2")
+
+    os.makedirs(subdir0)
+
+    try:
+        os.symlink(subdir0, subdir1)
+        os.symlink(subdir0, subdir2)
+    except OSError:
+        if WIN:
+            pytest.xfail("Ran on win without admin permissions")
+        else:
+            raise
+
+    fs = LocalFileSystem()
+    assert fs.info(subdir0)["type"] == "directory"
+    assert fs.info(subdir1)["type"] == "directory"
+    assert fs.info(subdir2)["type"] == "directory"
+
+    assert not fs.info(subdir0)["islink"]
+    assert fs.info(subdir1)["islink"]
+    assert fs.info(subdir2)["islink"]
+
+
 def test_isfilestore():
     fs = LocalFileSystem(auto_mkdir=False)
     assert fs._isfilestore()
@@ -506,13 +599,36 @@ def test_pickle(tmpdir):
     with pytest.raises(ValueError):
         pickle.dumps(f)
 
+    # with context
+    with fs.open(fn0, "rb") as f:
+        f.seek(1)
+        f2 = pickle.loads(pickle.dumps(f))
+        assert f2.tell() == 1
+        assert f2.read() == f.read()
+
+    # with fsspec.open https://github.com/fsspec/filesystem_spec/issues/579
+    with fsspec.open(fn0, "rb") as f:
+        f.seek(1)
+        f2 = pickle.loads(pickle.dumps(f))
+        assert f2.tell() == 1
+        assert f2.read() == f.read()
+
 
 def test_strip_protocol_expanduser():
-    path = "file://~\\foo\\bar" if sys.platform == "win32" else "file://~/foo/bar"
+    path = "file://~\\foo\\bar" if WIN else "file://~/foo/bar"
     stripped = LocalFileSystem._strip_protocol(path)
     assert path != stripped
     assert "file://" not in stripped
     assert stripped.startswith(os.path.expanduser("~").replace("\\", "/"))
+    assert not LocalFileSystem._strip_protocol("./").endswith("/")
+
+
+def test_mkdir_twice_faile(tmpdir):
+    fn = os.path.join(tmpdir, "test")
+    fs = fsspec.filesystem("file")
+    fs.mkdir(fn)
+    with pytest.raises(FileExistsError):
+        fs.mkdir(fn)
 
 
 def test_iterable(tmpdir):
@@ -594,3 +710,78 @@ def test_copy_errors(tmpdir):
             make_path_posix(os.path.join(dest2, "afile1")),
             make_path_posix(os.path.join(dest2, "afile2")),
         ]
+
+
+def test_transaction(tmpdir):
+    file = str(tmpdir / "test.txt")
+    fs = LocalFileSystem()
+
+    with fs.transaction:
+        content = "hello world"
+        with fs.open(file, "w") as fp:
+            fp.write(content)
+
+    with fs.open(file, "r") as fp:
+        read_content = fp.read()
+
+    assert content == read_content
+
+
+def test_delete_cwd(tmpdir):
+    cwd = os.getcwd()
+    fs = LocalFileSystem()
+    try:
+        os.chdir(tmpdir)
+        with pytest.raises(ValueError):
+            fs.rm(".", recursive=True)
+    finally:
+        os.chdir(cwd)
+
+
+@pytest.mark.parametrize(
+    "opener, ext", [(bz2.open, ".bz2"), (gzip.open, ".gz"), (open, "")]
+)
+def test_infer_compression(tmpdir, opener, ext):
+    filename = str(tmpdir / f"test{ext}")
+    content = b"hello world"
+    with opener(filename, "wb") as fp:
+        fp.write(content)
+
+    fs = LocalFileSystem()
+    with fs.open(f"file://{filename}", "rb", compression="infer") as fp:
+        read_content = fp.read()
+
+    assert content == read_content
+
+
+def test_info_path_like(tmpdir):
+    path = Path(tmpdir / "test_info")
+    path.write_text("fsspec")
+
+    fs = LocalFileSystem()
+    assert fs.exists(path)
+
+
+def test_seekable(tmpdir):
+    fs = LocalFileSystem()
+    tmpdir = str(tmpdir)
+    fn0 = os.path.join(tmpdir, "target")
+
+    with open(fn0, "wb") as f:
+        f.write(b"data")
+
+    f = fs.open(fn0, "rt")
+    assert f.seekable(), "file is not seekable"
+    f.seek(1)
+    assert f.read(1) == "a"
+    assert f.tell() == 2
+
+
+def test_numpy_fromfile(tmpdir):
+    # Regression test for #1005.
+    np = pytest.importorskip("numpy")
+    fn = str(tmpdir / "test_arr.npy")
+    dt = np.int64
+    arr = np.arange(10, dtype=dt)
+    arr.tofile(fn)
+    assert np.array_equal(np.fromfile(fn, dtype=dt), arr)
